@@ -3,14 +3,12 @@ use strict;
 use warnings;
 use lib qw(lib ../lib/ ../../hg_Gtk2-Ex-DbLinker-DbTools/lib/);
 use Gtk2 -init;
-
-#use lib "../lib/";
 use Log::Log4perl;
 Log::Log4perl->init("log_ex2.conf");
-
-use Forms::Langues2_sqla;
-
+use Forms::Langues2;
+use DataAccess::Sqla::Service;
 use DBI;
+#use Devel::Cycle;
 
 my $dbfile = "./data/ex1_1";
 
@@ -22,13 +20,18 @@ my $dbh = DBI->connect(
         PrintError => 1,
     }
 ) or die $DBI::errstr;
-
+my $f;
 sub load_main_w {
-
-    Forms::Langues2_sqla->new( { gladefolder => "./Forms", dbh => $dbh } );
+ my $data = DataAccess::Sqla::Service->new({dbh => $dbh }); 
+    $f = Forms::Langues2->new( { gladefolder => "./gladefiles", data_broker => $data } );
+    #find_cycle($f);
 
 }
 
 &load_main_w;
 Gtk2->main;
 
+#find_cycle($f);
+#print "Weakened\n";
+#find_weakened_cycle($f);
+
