@@ -1,4 +1,5 @@
 package Gtk2::Ex::DbLinker::Form;
+use Class::InsideOut qw(public private register id);
 use Gtk2::Ex::DbLinker;
 our $VERSION = $Gtk2::Ex::DbLinker::VERSION;
 
@@ -77,13 +78,20 @@ my %getter = (
 
 );
 
+
+
+private log => my %log;
+private event => my %events;
+private states => my %states;
+private widgets => my %widgets;
+
 # 	'GtkSpinButton' => \&set_spinbutton,
 # sub {return shift->child->get_text; },
 # sub {my $c = shift; print "getter_cbe\n"; my $iter = $c->get_active_iter; return $c->get_model->get( $iter ); }
 #
 sub new {
     my $class = shift;
-
+   
     #my ($class, $req)=@_;
     my %def = (
         null_string     => "null",
@@ -91,54 +99,100 @@ sub new {
         status_label    => "lbl_RecordStatus",
         rec_count_label => "lbl_RecordCount",
         locale          => "fr_CH",
-        auto_apply      => 1
+        auto_apply      => 1,
+        data_lock   => 0,
     );
 
     my %arg = ( ref $_[0] eq "HASH" ? ( %def, %{ $_[0] } ) : ( %def, @_ ) );
 
-    my $self = $class->SUPER::new(
+    #   @$self{qw(dman cols)} = delete @arg{qw(data_manager datawidgets)};
+    #   @$self{ keys %arg } = values(%arg);
+   
+
+my $self = $class->SUPER::new(
         childclass           => __PACKAGE__,
+        data_manager => $arg{data_manager},
+        builder => $arg{builder},
+        datawidgets => $arg{datawidgets},
+        on_current => $arg{on_current},
+        date_formatters => $arg{date_formatters},
+        time_zone => $arg{time_zone},
+        locale => $arg{locale},
         rec_spinner_callback => sub {
             my $self = shift;
-            return unless $self->{rec_spinner};
-            $self->{rec_spinner}
-              ->signal_handler_block( $self->{rs_value_changed_signal} );
-            $self->{rec_spinner}->set_value( $self->{pos} + 1 );
-            $self->{rec_spinner}
-              ->signal_handler_unblock( $self->{rs_value_changed_signal} );
+            my $id = id $self;
+            return unless $widgets{ $id}->{rec_spinner};
+            $widgets{ $id }->{rec_spinner}
+              ->signal_handler_block( $events{ $id }->{rs_value_changed_signal} );
+            $widgets{ $id }->{rec_spinner}->set_value( $self->_pos + 1 );
+            $widgets{ $id }->{rec_spinner}
+              ->signal_handler_unblock( $events{ $id }->{rs_value_changed_signal} );
 
         },
         rec_spinner_insert_callback => sub {
             my ( $self, $new_pos ) = @_;
-            return unless $self->{rec_spinner};
-            $self->{rec_spinner}
-              ->signal_handler_block( $self->{rs_value_changed_signal} );
-            my $first = ( $self->{pos} < 0 ? 0 : 1 );
-            $self->{rec_spinner}->set_range( $first, $new_pos + 1 );
-            $self->{rec_spinner}->set_value( $new_pos + 1 );
-            $self->{rec_spinner}
-              ->signal_handler_unblock( $self->{rs_value_changed_signal} );
+            my $id = id $self;
+            return unless $widgets{ $id }->{rec_spinner};
+            $widgets{ $id }->{rec_spinner}
+              ->signal_handler_block( $events{ $id }->{rs_value_changed_signal} );
+            my $first = ( $self->_pos < 0 ? 0 : 1 );
+            $widgets{ $id }->{rec_spinner}->set_range( $first, $new_pos + 1 );
+            $widgets{ $id }->{rec_spinner}->set_value( $new_pos + 1 );
+            $widgets{ $id }->{rec_spinner}
+              ->signal_handler_unblock( $events{ $id }->{rs_value_changed_signal} );
 
         },
 
     );
-    @$self{qw(dman cols)} = delete @arg{qw(data_manager datawidgets)};
-    @$self{ keys %arg } = values(%arg);
-    bless $self, $class;
+    register $self;
+     my $ido = id $self;
+    
+    delete @arg{ $self->_super_args_needed };
+     my $arg_value_ref = \%arg;
+
+   my $arg_holder_ref = { 
+        rec_count_label => \$widgets{ $ido}->{rec_count_label},
+        status_label => \$widgets{ $ido}->{status_label},
+        rec_spinner => \$widgets{ $ido}->{rec_spinner},
+        null_string => \$widgets{ $ido }->{ null_string },
+        data_lock => \$events{ $ido }->{data_lock},
+
+    } ;
+    for my $name (keys %{$arg_holder_ref}){
+        next unless defined ($arg{$name});
+
+        if (ref $arg_holder_ref->{$name} eq "HASH"){
+           $arg_holder_ref->{$name}->{ $ido } = $arg_value_ref->{$name};
+
+        } 
+        #elsif (ref $arg_holder_ref->{$name} eq "ARRAY") {
+
+        #} 
+        else {
+            ${ $arg_holder_ref->{$name} } =  $arg{$name};
+        
+        }
+    
+    }
+   
+
+    #bless $self, $class;
 
     #$self->{cols} = [];
     $self->_init;
     $self->SUPER::_init;
-
+    #$log{$ido}->debug("\%arg ", Dumper %arg);
+    #$log{$ido}->debug("\%widgets ", Dumper %widgets);
+=for comment
     my @dates;
 
-    $self->{subform} = [];
+    #$self->{subform} = [];
 
     #my %formatters_db;
     #my %formatters_f;
     # $self->{dates_formatted} = \(keys %{$self->{date_formatters}});
     foreach my $v ( keys %{ $self->{date_formatters} } ) {
-        $self->{log}->debug( "** " . $v . " **" );
+        $log{ $id }->debug( "** " . $v . " **" );
         push @dates, $v;
     }
     $self->{dates_formatted} = \@dates;
@@ -147,7 +201,7 @@ sub new {
     $self->{dates_formatters} = {};
     $self->{inserting}        = 0;
     $self->{pos2del}          = [];
-
+=cut
     return $self;
 
 }    #new
@@ -158,38 +212,41 @@ sub _get_getter { my $self = shift; return %getter; }
 sub _init {
 
     my ($self) = @_;
-    $self->{painting} = 1;
+    my $id = id $self;
+    $self->_painting(1);
 
 # get a ref to the Gtk widget used for the record spinner or if the id has been guiven, get the ref via the builder
-    $self->{rec_spinner} = (
-        ref $self->{rec_spinner}
-        ? $self->{rec_spinner}
-        : $self->{builder}->get_object( $self->{rec_spinner} ) );
-     if ($self->{rec_spinner}){
-  	$self->{rec_spinner}->configure(Gtk2::Adjustment->new(1.0, 1.0, 10.0, 1.0, 10.0, 0.0), 0.1, 0);
+    $widgets{ $id }->{rec_spinner} = (
+        ref $widgets{ $id }->{rec_spinner}
+        ? $widgets{ $id }->{rec_spinner}
+        : $self->_builder()->get_object( $widgets{ $id }->{rec_spinner} ) );
+     if ($widgets{ $id }->{rec_spinner}){
+  	$widgets{ $id }->{rec_spinner}->configure(Gtk2::Adjustment->new(1.0, 1.0, 10.0, 1.0, 10.0, 0.0), 0.1, 0);
      }
-    $self->{rec_count_label} = (
-        ref $self->{rec_count_label}
-        ? $self->{rec_count_label}
-        : $self->{builder}->get_object( $self->{rec_count_label} ) );
-    $self->{status_label} = (
-        ref $self->{status_label}
-        ? $self->{status_label}
-        : $self->{builder}->get_object( $self->{status_label} ) );
+    $widgets{ $id }->{rec_count_label} = (
+        ref $widgets{ $id }->{rec_count_label}
+        ? $widgets{ $id }->{rec_count_label}
+        : $self->_builder()->get_object( $widgets{ $id }->{rec_count_label} ) );
+    $widgets{ $id }->{status_label} = (
+        ref $widgets{ $id }->{status_label}
+        ? $widgets{ $id }->{status_label}
+        : $self->_builder->get_object( $widgets{ $id }->{status_label} ) );
 
-    $self->{log} = Log::Log4perl->get_logger(__PACKAGE__);
-    $self->{log}->debug(" ** New Form object ** ");
-    $self->{changed} = 0;
+    $log{ $id } = Log::Log4perl->get_logger(__PACKAGE__);
+    $log{ $id }->debug(" ** New Form object ** ");
+    $self->_changed(0);
+=for comment
     if ( !defined $self->{cols} ) {
         my @col = $self->{dman}->get_field_names;
         $self->{cols} = \@col;
     }
-    $self->{log}->debug( "_init: cols ", Dumper @{ $self->{cols} } );
+=cut
+    $log{ $id }->debug( "_init: cols ", Dumper @{ $self->_cols  } );
     $self->_bind_on_changed;
     $self->_set_recordspinner;
     croak __PACKAGE__ . ": a data manager is required"
-      unless ( defined $self->{dman} );
-    $self->{dman}->set_row_pos(0);
+      unless ( defined $self->_dman );
+    $self->_dman->set_row_pos(0);
 }
 
 #add the values contained in the array @$aref in the combo $name
@@ -197,6 +254,7 @@ sub _init {
 
 sub add_combo_values {
     my ( $self, $w, $aref ) = @_;
+        my $id = id $self;
     my $wref       = ref $w;
     my @supp_class = (qw/Gtk2::ComboBox Gtk2::ComboBoxEntry/);
     my %supported  = map { $_ => 1 } @supp_class;
@@ -235,7 +293,8 @@ sub add_combo_values {
 sub add_combo {
 
     #my ($self, $req)=@_;
-    my $self = shift;
+    my $class = shift;
+        my $id = id $class;
     my %h;
     my $req = ( ref $_[0] eq "HASH" ) ? $_[0] : ( %h = (@_) ) && \%h;
     my $combo = {
@@ -254,28 +313,33 @@ sub add_combo {
 
     }
     my @list_def;
-    if ( $$req{builder} && ( ref $self eq "" ) ) {    #static init
+    my $self;
+    if ( $$req{builder} && ( ref $class eq "" ) ) {    #static init
 
         $self = {};
-        $self->{builder} = $$req{builder};
+        bless $self, $class;
+        $self->_builder( $$req{builder} );
 
-        $self->{log} = Log::Log4perl->get_logger("Gtk2::Ex::DbLinker::Form");
+        $log{ $id } = Log::Log4perl->get_logger("Gtk2::Ex::DbLinker::Form");
 
-        my $w = $self->{builder}->get_object( $combo->{id} );
+        my $w = $self->_builder->get_object( $combo->{id} );
         if ($w) {
             my $name = $w->get_name;
-            $self->{datawidgets}->{ $combo->{id} }     = $w;
-            $self->{datawidgetsName}->{ $combo->{id} } = $name;
+            $self->_datawidgets( $combo->{id} , $w);
+            $self->_datawidgetsName( $combo->{id},  $name);
         } else {
             croak "no widget found for combo " . $combo->{id};
         }
+    } 
+    else {
+        $self = $class;
     }
 
-    $self->{log}->debug( "cols: " . join( " ", @cols ) );
-    $self->{log}->debug( Dumper $self->{datawidgets} );
-    my $w = $self->{datawidgets}->{ $combo->{id} };
+    $log{ $id }->debug( "cols: " . join( " ", @cols ) );
+    $log{ $id }->debug( Dumper $self->_datawidgets );
+    my $w = $self->_datawidgets( $combo->{id} );
 
-    # $self->{log}->debug( Dumper $self->{datawidgets});
+    # $log{ $id }->debug( Dumper $self->{datawidgets});
     # die Dumper $w;
     croak( 'no widget found for combo ' . $combo->{id} ) unless ($w);
 
@@ -297,14 +361,14 @@ sub add_combo {
         my $gtype = $fieldtype{$type} if ($type);
         if ($gtype) {
 
-            $self->{log}->debug( "field: " . $field . " type : " . $type );
+            $log{ $id }->debug( "field: " . $field . " type : " . $type );
         } else {
-            $self->{log}->debug(
+            $log{ $id }->debug(
                 "no Glib type found for field $field assuming Glib::String");
             $gtype = "Glib::String";
         }
         push @list_def, $gtype;
-        $self->{log}->debug("add_combo: $field $column_no $gtype");
+        $log{ $id }->debug("add_combo: $field $column_no $gtype");
 
         #column 0 is not shown unless it's the only column
 
@@ -312,11 +376,11 @@ sub add_combo {
         #column above 1 are displayed here:
         if ( ( !defined $model ) && $column_no > 1 ) {
 
-            $self->{log}->debug("new renderer for $column_no");
+            $log{ $id }->debug("new renderer for $column_no");
             my $renderer = Gtk2::CellRendererText->new;
             $w->pack_start( $renderer, TRUE );
 
-  # $self->{log}->debug("add_combo: " . $field . " set text for " . $column_no);
+  # $log{ $id }->debug("add_combo: " . $field . " set text for " . $column_no);
             $w->set_attributes( $renderer, 'text' => $column_no );
 
             #$combo->{renderers_setup} = 1;
@@ -328,14 +392,14 @@ sub add_combo {
          #$combo->{renderers_setup} = 1;
 
     if ($model) {
-        $self->{log}->debug("clearing existing model");
+        $log{ $id }->debug("clearing existing model");
         $model->clear;
 
     } else {
         $model = Gtk2::ListStore->new(@list_def);
         $w->set_model($model);
     }
-    $self->{log}->debug( join( " ", @list_def ) );
+    $log{ $id }->debug( join( " ", @list_def ) );
     my $i;
     my $last = $combo->{dman}->row_count - 1;
 
@@ -353,17 +417,17 @@ sub add_combo {
             #push @model_row, $column, $row->{$field};
             my $value = $combo->{dman}->get_field($field);
 
-            #$self->{log}->debug("add_combo: " . $value);
+            #$log{ $id }->debug("add_combo: " . $value);
             push @model_row, $column++, $value;
 
             #$column ++;
         }
-	#$self->{log}->debug( "row : " . join( " ", @model_row ) );
+	#$log{ $id }->debug( "row : " . join( " ", @model_row ) );
         $model->set(@model_row);
     }
-    $self->{log}->debug( "add_combo: " . $i . " rows added" );
+    $log{ $id }->debug( "add_combo: " . $i . " rows added" );
 
-    if ( $self->{datawidgetsName}->{ $combo->{id} } eq "GtkComboBoxEntry" ) {
+    if ( $self->_datawidgetsName( $combo->{id} ) eq "GtkComboBoxEntry" ) {
 
         #if ( ! $self->{combos_set}->{$combo->{id}} ) {
         #$w->set_text_column( 1 );
@@ -382,26 +446,26 @@ sub add_combo {
 #bind an onchanged sub with each modification of the datafields
 sub _bind_on_changed {
     my $self = shift;
-
+        my $idob = id $self;
     # my @cols = $self->{dman}->get_field_names;
-    foreach my $id ( @{ $self->{cols} } ) {
-        my $w = $self->{builder}->get_object($id);
-        $self->{log}->debug( "bind_on_changed looking for widget " . $id );
+    foreach my $id ( @{ $self->_cols } ) {
+        my $w = $self->_builder->get_object($id);
+        $log{ $idob }->debug( "bind_on_changed looking for widget " . $id );
         if ($w) {
             my $name = $w->get_name;
-            $self->{datawidgets}->{$id}     = $w;
-            $self->{datawidgetsName}->{$id} = $name;
+            $self->_datawidgets($id, $w);
+            $self->_datawidgetsName($id, $name);
             if ( ref( $signals{$name} ) eq "CODE" ) {
                 my $coderef = $signals{$name};
                 $w = &$coderef( $self, $w );
                 $name = ref $w;
 
             }
-            $self->{log}->debug("bind  $name $id with self->changed \n");
+            $log{ $idob }->debug("bind  $name $id with self->changed \n");
             $w->signal_connect_after(
-                $signals{$name} => sub { $self->_changed($id) } );
+                $signals{$name} => sub { $self->_change_values($id) } );
         } else {
-            $self->{log}->debug(" ... not found ");
+            $log{ $idob }->debug(" ... not found ");
         }
     }
 
@@ -411,69 +475,71 @@ sub _bind_on_changed {
 # Place
 sub _set_recordspinner {
     my $self = shift;
-    $self->{log}->debug("set_recordspinner");
+        my $id = id $self;
+    $log{ $id }->debug("set_recordspinner");
 
-    # die unless($self->{rec_spinner});
+    # die unless($widgets{ $id }->{rec_spinner});
     my $coderef;
-    if ( $self->{rec_spinner} ) {
+    if ( $widgets{ $id }->{rec_spinner} ) {
 
 #	    The return type of the signal_connect() function is a tag that identifies your callback function.
 #	    You may have as many callbacks per signal and per object as you need, and each will be executed in turn,
 #	    in the order they were attached.
-        $coderef = $self->{rec_spinner}->signal_connect_after(
+        $coderef = $widgets{ $id }->{rec_spinner}->signal_connect_after(
             value_changed => sub {
-                my $pos = $self->{rec_spinner}->get_text - 1;
-                $self->{log}->debug( "rs_value changed will move to " . $pos );
-                $self->{rec_spinner}->signal_handler_block($coderef);
+                my $pos = $widgets{ $id }->{rec_spinner}->get_text - 1;
+                $log{ $id }->debug( "rs_value changed will move to " . $pos );
+                $widgets{ $id }->{rec_spinner}->signal_handler_block($coderef);
 
                 #$self->move( undef, $pos);
-                if ( $self->{auto_apply} && $self->has_changed ) {
+                if ( $self->_auto_apply && $self->has_changed ) {
                     $self->apply;
                 }
 
                 #done in _display_data
                 #$self->{dman}->set_row_pos($pos);
                 $self->_display_data($pos);
-                $self->{rec_spinner}->signal_handler_unblock($coderef);
+                $widgets{ $id }->{rec_spinner}->signal_handler_unblock($coderef);
                 return TRUE;
             }
         );
-        $self->{rs_value_changed_signal} = $coderef;
-        $self->{log}->debug("recordspinner set");
+        $events{ $id }->{rs_value_changed_signal} = $coderef;
+        $log{ $id }->debug("recordspinner set");
     }
 
 }
 
 sub _set_rs_range {
     my ( $self, $first, $last ) = @_;
-
+    my $id = id $self;
     # Convenience function that sets the min / max value of the record spinner
-    $self->{log}->debug( "set_rs_range  first : " . $first );
-    if ( $self->{rec_spinner} ) {
-        my $ad = $self->{rec_spinner}->get_adjustment;
-        $self->{log}->debug( "adj lower : " . $ad->lower );
+    $log{ $id }->debug( "set_rs_range  first : " . $first );
+    if ( $widgets{ $id }->{rec_spinner} ) {
+        my $ad = $widgets{ $id }->{rec_spinner}->get_adjustment;
+        $log{ $id }->debug( "adj lower : " . $ad->lower );
         if ( $first < $ad->lower ) {
             $ad->lower($first);
-            $self->{rec_spinner}->set_adjustment($ad);
+            $widgets{ $id }->{rec_spinner}->set_adjustment($ad);
         }
-        $self->{rec_spinner}
-          ->signal_handler_block( $self->{rs_value_changed_signal} );
-        $self->{rec_spinner}->set_range( $first, $last );
-        $self->{rec_spinner}
-          ->signal_handler_unblock( $self->{rs_value_changed_signal} );
+        $widgets{ $id }->{rec_spinner}
+          ->signal_handler_block( $events{ $id }->{rs_value_changed_signal} );
+        $widgets{ $id }->{rec_spinner}->set_range( $first, $last );
+        $widgets{ $id }->{rec_spinner}
+          ->signal_handler_unblock( $events{ $id }->{rs_value_changed_signal} );
     }
-    $self->{rec_count_label}->set_text( " / " . $self->{dman}->row_count );
+    $widgets{ $id }->{rec_count_label}->set_text( " / " . $self->_dman->row_count );
     return TRUE;
 
 }
 
 sub _set_entry {
     my ( $self, $w, $x ) = @_;
+        my $id = id $self;
     if ( defined $x ) {
-        $self->{log}->debug( "set_entry: " . $x );
+        $log{ $id }->debug( "set_entry: " . $x );
         $w->set_text($x);
     } else {
-        $self->{log}->debug( "set_entry: text entry undef " . $w->get_name );
+        $log{ $id }->debug( "set_entry: text entry undef " . $w->get_name );
         $w->set_text("");
     }
 
@@ -481,14 +547,16 @@ sub _set_entry {
 
 sub _set_textentry {
     my ( $self, $w, $x ) = @_;
-    $self->{log}->debug("set_textentry text entry undef") if ( !defined $x );
+        my $id = id $self;
+    $log{ $id }->debug("set_textentry text entry undef") if ( !defined $x );
     $w->get_buffer->set_text( $x || "" );
 
 }
 
 sub _set_combo {
     my ( $self, $w, $x ) = @_;
-    $self->{log}->debug( "set_combo value "
+        my $id = id $self;
+    $log{ $id }->debug( "set_combo value "
           . ( defined $x ? $x : " undef" )
           . " widget: "
           . ref $w );
@@ -510,31 +578,34 @@ sub _set_combo {
         $iter = $m->iter_next($iter);
     }
     if ( !$match_found && $x ) {
-        $self->{log}->debug( "Failed to set " . ref $w . " to $x\n" );
+        $log{ $id }->debug( "Failed to set " . ref $w . " to $x\n" );
     }
 
 }
 
 sub _set_check {
     my ( $self, $w, $x ) = @_;
+        my $id = id $self;
     $w->set_active( ( defined $x ? $x : 0 ) );
 }
 
 sub _get_combobox_firstvalue {
     my ( $self, $c ) = @_;
-
+    my $id = id $self;
+    
     #print "getter_cb\n";
     my $iter = $c->get_active_iter;
     unless ($iter) {
-        $self->{log}->debug( "iter undef row: " . $c->get_active );
+        $log{ $id }->debug( "iter undef row: " . $c->get_active );
     }
     return ( $iter ? $c->get_model->get( $iter, 0 ) : undef );
 }
 
 sub _set_spinbutton {
     my ( $self, $w, $x ) = @_;
-    if ( $self->getID($w) eq $self->getID( $self->{rec_spinner} ) ) {
-        $self->{log}->debug("Found record_spinner... leaving");
+        my $id = id $self;
+    if ( $self->getID($w) eq $self->getID( $widgets{ $id }->{rec_spinner} ) ) {
+        $log{ $id }->debug("Found record_spinner... leaving");
         return;
     }
     $w->set_value( $x || 0 );
@@ -543,18 +614,19 @@ sub _set_spinbutton {
 
 sub _get_textbuffer {
     my ( $self, $w ) = @_;
+        my $id = id $self;
     return $w->get_buffer;
 
 }
 
-sub _changed {
+sub _change_values {
     my ( $self, $fieldname ) = @_;
-
-    # $self->{log}->debug("self->changed for $fieldname");
-    if ( !$self->{painting} ) {
-        $self->{changed} = 1;
-        if ( $self->{on_changed} ) {
-            $self->{on_changed}();
+    my $id = id $self;
+    # $log{ $id }->debug("self->changed for $fieldname");
+    if ( !$self->_painting ) {
+        $self->_changed(1);
+        if ( $events{ $id }->{on_change} ) {
+            $events{ $id }->{on_change}();
         }
         $self->_set_record_status_label;
     }
@@ -565,25 +637,27 @@ sub _changed {
 sub _set_record_status_label {
 
     my $self = shift;
+    my $id = id $self;
+  $log{ $id }->debug("set_record_satus_label changed is " . $self->_changed);
 
- # $self->{log}->debug("set_record_satus_label changed is " . $self->{changed});
-
-    if ( $self->{status_label} ) {
-        if ( $self->{data_lock} ) {
-            $self->{status_label}
+    if ( $widgets{ $id }->{status_label} ) {
+        if ( $events{ $id }->{data_lock} ) {
+            $widgets{ $id }->{status_label}
               ->set_markup("<b><i><span color='red'>Locked</span></i></b>");
-        } elsif ( $self->{changed} ) {
+        } elsif ( $self->_changed ) {
 
-            $self->{status_label}
+            $widgets{ $id }->{status_label}
               ->set_markup("<b><span color='red'>Changed</span></b>");
 
         } else {
-            $self->{status_label}
+            $widgets{ $id }->{status_label}
               ->set_markup("<b><span color='blue'>Synchronized</span></b>");
         }
     }
 }
 
+=for comment
+
 #parameter $in_db is 0 or 1 :
 # 0 we are reading from the db, and the format to use are at the pos 0 and 1 in the array of format for the field
 # 1 we are writing to the db and the format are to use in a revers order
@@ -591,21 +665,22 @@ sub _set_record_status_label {
 # $v the date string from the form (if in_db is 1) or from the db (if in_db is 0)
 sub _format_date {
     my ( $self, $in_db, $id, $v ) = @_;
-    $self->{log}->debug( "format_date received date: " . $v );
+        my $id = id $self;
+    $log{ $id }->debug( "format_date received date: " . $v );
     my ( $pos1, $pos2 ) = ( $in_db ? ( 1, 0 ) : ( 0, 1 ) );
     my $format = $self->{date_formatters}->{$id}->[$pos1];
     my $f      = $self->_get_dateformatter($format);
     my $dt     = $f->parse_datetime($v) or croak( $f->errmsg );
-    $self->{log}->debug( "format_date:  date time object ymd: " . $dt->ymd );
+    $log{ $id }->debug( "format_date:  date time object ymd: " . $dt->ymd );
     $format = $self->{date_formatters}->{$id}->[$pos2];
     $f      = $self->_get_dateformatter($format);
     my $r = $f->format_datetime($dt) or croak( $f->errmsg );
-    $self->{log}->debug( "format_date formatted date: " . $r );
+    $log{ $id }->debug( "format_date formatted date: " . $r );
 
     return $r;
 
 }
-
+=cut
 1;
 
 __END__
@@ -678,7 +753,7 @@ C<time_zone> and C<locale> are needed by Date::Time::Strptime.
 
 
 
-To display new rows on a bound subform, connect the on_changed event to the field of the primary key in the main form.
+To display new rows on a bound subform, connect the on_change event to the field of the primary key in the main form.
 In this sub, call a sub to synchonize the form:
 
 In the main form:
